@@ -102,6 +102,7 @@ cmdhandler_handle_cmd_help(int sockfd)
                                     "configurations.\n"
         "update [--all]              Update zone list and all signer "
                                     "configurations.\n"
+        "retransfer <zone>           Retransfer the zone from the master.\n"
         "start                       Start the engine.\n"
         "running                     Check if the engine is running.\n"
         "reload                      Reload the engine.\n"
@@ -249,6 +250,50 @@ cmdhandler_handle_cmd_update(int sockfd, cmdhandler_type* cmdc,
 }
 
 
+/**
+ * Handle the 'retransfer' command.
+ *
+ */
+static void
+cmdhandler_handle_cmd_retransfer(int sockfd, cmdhandler_type* cmdc, char* tbd)
+{
+    engine_type* engine = NULL;
+    char buf[ODS_SE_MAXLINE];
+    zone_type* zone = NULL;
+    ods_log_assert(tbd);
+    ods_log_assert(cmdc);
+    ods_log_assert(cmdc->engine);
+    engine = (engine_type*) cmdc->engine;
+    ods_log_assert(engine->taskq);
+    /* look up zone */
+    lock_basic_lock(&engine->zonelist->zl_lock);
+    zone = zonelist_lookup_zone_by_name(engine->zonelist, tbd,
+        LDNS_RR_CLASS_IN);
+    /* If this zone is just added, don't retransfer (it might not have a
+     * task yet) */
+    if (zone && zone->zl_status == ZONE_ZL_ADDED) {
+        zone = NULL;
+    }
+    lock_basic_unlock(&engine->zonelist->zl_lock);
+
+    if (!zone) {
+        (void)snprintf(buf, ODS_SE_MAXLINE, "Error: Zone %s not found.\n",
+            tbd);
+        ods_writen(sockfd, buf, strlen(buf));
+        return;
+    }
+    zone->xfrd->serial_retransfer = 1;
+    xfrd_set_timer_now(zone->xfrd);
+    ods_log_debug("[%s] forward a notify", cmdh_str);
+    dnshandler_fwd_notify(engine->dnshandler,
+        (uint8_t*) ODS_SE_NOTIFY_CMD, strlen(ODS_SE_NOTIFY_CMD));
+    (void)snprintf(buf, ODS_SE_MAXLINE, "Zone %s being retransferred.\n", tbd);
+    ods_writen(sockfd, buf, strlen(buf));
+    ods_log_verbose("[%s] zone %s being retransferred", cmdh_str, tbd);
+    return;
+}
+
+
 static uint32_t
 max(uint32_t a, uint32_t b)
 {
@@ -395,7 +440,6 @@ cmdhandler_handle_cmd_clear(int sockfd, cmdhandler_type* cmdc, const char* tbd)
     engine_type* engine = NULL;
     char buf[ODS_SE_MAXLINE];
     zone_type* zone = NULL;
-    task_type* task = NULL;
     uint32_t inbserial = 0;
     uint32_t intserial = 0;
     uint32_t outserial = 0;
@@ -711,7 +755,7 @@ again:
             cmdhandler_handle_cmd_zones(sockfd, cmdc);
         } else if (n >= 4 && strncmp(buf, "sign", 4) == 0) {
             ods_log_debug("[%s] sign zone command", cmdh_str);
-            if (buf[4] == '\0') {
+            if (n == 4 || buf[4] == '\0') {
                 /* NOTE: wouldn't it be nice that we default to --all? */
                 cmdhandler_handle_cmd_error(sockfd, "sign command needs "
                     "an argument (either '--all' or a zone name)");
@@ -722,7 +766,7 @@ again:
             }
         } else if (n >= 5 && strncmp(buf, "clear", 5) == 0) {
             ods_log_debug("[%s] clear zone command", cmdh_str);
-            if (buf[5] == '\0') {
+            if (n == 5 || buf[5] == '\0') {
                 cmdhandler_handle_cmd_error(sockfd, "clear command needs "
                     "a zone name");
             } else if (buf[5] != ' ') {
@@ -738,7 +782,7 @@ again:
             cmdhandler_handle_cmd_flush(sockfd, cmdc);
         } else if (n >= 6 && strncmp(buf, "update", 6) == 0) {
             ods_log_debug("[%s] update command", cmdh_str);
-            if (buf[6] == '\0') {
+            if (n == 6 || buf[6] == '\0') {
                 cmdhandler_handle_cmd_update(sockfd, cmdc, "--all");
             } else if (buf[6] != ' ') {
                 cmdhandler_handle_cmd_unknown(sockfd, buf);
@@ -760,7 +804,7 @@ again:
             cmdhandler_handle_cmd_running(sockfd);
         } else if (n >= 9 && strncmp(buf, "verbosity", 9) == 0) {
             ods_log_debug("[%s] verbosity command", cmdh_str);
-            if (buf[9] == '\0') {
+            if (n == 9 || buf[9] == '\0') {
                 cmdhandler_handle_cmd_error(sockfd, "verbosity command "
                     "an argument (verbosity level)");
             } else if (buf[9] != ' ') {
@@ -768,6 +812,16 @@ again:
             } else {
                 cmdhandler_handle_cmd_verbosity(sockfd, cmdc, atoi(&buf[10]));
             }
+        } else if (n >= 10 && strncmp(buf, "retransfer", 10) == 0) {
+            ods_log_debug("[%s] retransfer zone command", cmdh_str);
+            if (n == 10 || buf[10] == '\0') {
+                cmdhandler_handle_cmd_error(sockfd, "retransfer command needs "
+                    "an argument (a zone name)");
+            } else if (buf[10] != ' ') {
+                cmdhandler_handle_cmd_unknown(sockfd, buf);
+            } else {
+                cmdhandler_handle_cmd_retransfer(sockfd, cmdc, &buf[11]);
+            }
         } else if (n > 0) {
             ods_log_debug("[%s] unknown command", cmdh_str);
             cmdhandler_handle_cmd_unknown(sockfd, buf);
