@@ -128,9 +128,14 @@ ods_build_path(const char* file, const char* suffix, int dir, int no_slash)
     size_t len_suffix = 0;
     size_t len_total = 0;
     char* openf = NULL;
+    char* f = "root";
 
     if (file) {
-        len_file = strlen(file);
+        if (ods_strcmp(file, ".")) {
+            f = file;
+        }
+
+        len_file = strlen(f);
         if (suffix) {
             len_suffix = strlen(suffix);
         }
@@ -146,7 +151,7 @@ ods_build_path(const char* file, const char* suffix, int dir, int no_slash)
                 return NULL;
             }
 
-            strncpy(openf, file, len_file);
+            strncpy(openf, f, len_file);
             openf[len_file] = '\0';
             if (no_slash) {
                 size_t i = 0;
